@@ -9,7 +9,7 @@ use URI;
 use Try::Tiny;
 use Carp;
 
-our $VERSION = "0.05";
+our $VERSION = "0.06";
 our $API_BASE = 'api.mailgun.net/v3';
 
 use Class::Accessor::Lite (
@@ -42,24 +42,31 @@ sub decode_response {
 }
 
 sub recursive {
-    my ($self, $method, $key) = @_;
+    my ($self, $method, $query, $key, $api_uri) = @_;
 
+    $query //= {};
     $key //= 'items';
-    my $query = '';
     my @result;
-
-    while (1) {
-        my $api_uri = URI->new($self->api_url($method));
-        $api_uri->query($query);
-        my $res = $self->client->get($api_uri->as_string);
-        my $json = $self->decode_response($res);
-        last unless $json && scalar @{$json->{$key}};
-        push @result, @{$json->{$key}};
-        my $next_uri = URI->new($json->{paging}->{next});
-        $query = $next_uri->query;
+    my $previous;
+    unless($api_uri) {
+        $api_uri = URI->new($self->api_url($method));
+        $api_uri->query_form($query);
     }
 
-    return \@result;
+    while (1) {
+        my $res = $self->client->get($api_uri->as_string);
+        my $json = $self->decode_response($res);
+        unless($json && scalar @{$json->{$key}}) {
+            $previous = URI->new($json->{paging}->{previous});
+            $previous->userinfo('api:'.$self->api_key);
+            last;
+        }
+        push @result, @{$json->{$key}};
+        $api_uri = URI->new($json->{paging}->{next});
+        $api_uri->userinfo('api:'.$self->api_key);
+    }
+
+    return \@result, $previous;
 }
 
 sub client {
@@ -175,7 +182,21 @@ sub delete_list_member {
 sub event {
     my ($self, $args) = @_;
 
-    my $res = $self->client->get($self->api_url("events"));
+    if (ref($args) && ref($args) eq 'HASH') {
+        return $self->recursive("events", $args);
+    } else {
+        return $self->recursive("events", {}, 'items', URI->new($args));
+    }
+}
+
+sub get_message_from_event {
+    my ($self, $event) = @_;
+
+    die "invalid event! this method need 'stored' event only." if $event->{event} ne 'stored';
+    my $uri = URI->new($event->{storage}->{url});
+    $uri->userinfo('api:'.$self->api_key);
+
+    my $res = $self->client->get($uri->as_string);
     $self->decode_response($res);
 }
 
@@ -376,6 +397,37 @@ Delete member for mailing list.
 
 L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
 
+=head2 event($option|$uri)
+
+Get event data.
+
+    # get event data
+    my ($events, $purl) = $mailgun->event({ event => 'stored' });
+
+L<Events|https://documentation.mailgun.com/api-events.html>
+
+=head2 get_message_from_event($event)
+
+Get stored message.
+
+    # get event data
+    my ($events, $purl) = $mailgun->event({ event => 'stored' });
+    my $msg = $mailgun->get_message_from_event($events->[0]);
+
+L<Stored Message|https://documentation.mailgun.com/api-sending.html#retrieving-stored-messages>
+
+=head1 Event Pooling
+
+event method return previous url. it can use for fetch event.
+
+    # event Pooling
+    my ($events, $purl) = $mailgun->event({ event => 'stored' });
+    // do something ...
+    $events = $mailgun->event($purl);
+    // ...
+
+L<Event Polling|https://documentation.mailgun.com/api-events.html#event-polling>    
+
 =head1 TODO
 
 this API not implement yet.
@@ -384,8 +436,6 @@ this API not implement yet.
 
 =item * L<Domains|https://documentation.mailgun.com/api-domains.html>
 
-=item * L<Events|https://documentation.mailgun.com/api-events.html>
-
 =item * L<Stats|https://documentation.mailgun.com/api-stats.html>
 
 =item * L<Tags|https://documentation.mailgun.com/api-tags.html>
